# MIT License
#
# Copyright (c) 2020 Evgeny Medvedev, evge.medvedev@gmail.com
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


from datetime import datetime

import click
from tezosetl.rpc.tezos_rpc import TezosRpc

from tezosetl.service.tezos_block_range_service import TezosBlockRangeService
from blockchainetl_common.file_utils import smart_open
from blockchainetl_common.logging_utils import logging_basic_config

logging_basic_config()


@click.command(context_settings=dict(help_option_names=['-h', '--help']))
@click.option('-p', '--provider-uri', default='https://mainnet-tezos.giganode.io', show_default=True, type=str,
              help='The URI of the remote Tezos node')
@click.option('-d', '--date', required=True, type=lambda d: datetime.strptime(d, '%Y-%m-%d'),
              help='The date e.g. 2018-01-01.')
@click.option('-o', '--output', default='-', type=str, help='The output file. If not specified stdout is used.')
def get_block_range_for_date(provider_uri, date, output):
    """Outputs start and end blocks for a given date."""

    tezos_rpc = TezosRpc(provider_uri)
    tezos_service = TezosBlockRangeService(tezos_rpc)

    start_block, end_block = tezos_service.get_block_range_for_date(date)

    with smart_open(output, 'w') as output_file:
        output_file.write('{},{}\n'.format(start_block, end_block))



class OperationKind:
    activate_account = 'activate_account'
    ballot = 'ballot'
    delegation = 'delegation'
    double_baking_evidence = 'double_baking_evidence'
    double_endorsement_evidence = 'double_endorsement_evidence'
    endorsement = 'endorsement'
    origination = 'origination'
    proposals = 'proposals'
    reveal = 'reveal'
    seed_nonce_revelation = 'seed_nonce_revelation'
    transaction = 'transaction'

    ALL = [
        activate_account,
        ballot,
        delegation,
        double_baking_evidence,
        double_endorsement_evidence,
        endorsement,
        origination,
        proposals,
        reveal,
        seed_nonce_revelation,
        transaction,
    ]



# MIT License
#
# Copyright (c) 2020 Evgeny Medvedev, evge.medvedev@gmail.com
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import logging
import os
from time import time

from blockchainetl_common.logging_utils import logging_basic_config
from blockchainetl_common.thread_local_proxy import ThreadLocalProxy
from tezosetl.jobs.export_job import ExportJob
from tezosetl.jobs.exporters.tezos_item_exporter import TezosItemExporter
from tezosetl.rpc.tezos_rpc import TezosRpc

logging_basic_config()
logger = logging.getLogger('export_partitioned')


def export_partitioned(partitions, output_dir, output_format, provider_uri, max_workers, batch_size):
    for batch_start_block, batch_end_block, partition_dir, *args in partitions:
        # # # start # # #

        start_time = time()

        padded_batch_start_block = str(batch_start_block).zfill(8)
        padded_batch_end_block = str(batch_end_block).zfill(8)
        block_range = '{padded_batch_start_block}-{padded_batch_end_block}'.format(
            padded_batch_start_block=padded_batch_start_block,
            padded_batch_end_block=padded_batch_end_block,
        )

        partition_output_dir = '{output_dir}/blocks{partition_dir}'.format(
            output_dir=output_dir,
            partition_dir=partition_dir,
        )
        os.makedirs(os.path.dirname(partition_output_dir), exist_ok=True)

        logger.info('Exporting blocks {block_range} to {partition_output_dir}'.format(
            block_range=block_range,
            partition_output_dir=partition_output_dir,
        ))

        job = ExportJob(
            start_block=batch_start_block,
            end_block=batch_end_block,
            batch_size=batch_size,
            tezos_rpc=ThreadLocalProxy(lambda: TezosRpc(provider_uri)),
            max_workers=max_workers,
            item_exporter=TezosItemExporter(partition_output_dir, output_format=output_format),
        )
        job.run()

        # # # finish # # #

        end_time = time()
        time_diff = round(end_time - start_time, 5)
        logger.info('Exporting blocks {block_range} took {time_diff} seconds'.format(
            block_range=block_range,
            time_diff=time_diff,
        ))



# MIT License
#
# Copyright (c) 2020 Evgeny Medvedev, evge.medvedev@gmail.com
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from tezosetl.utils.cast_utils import safe_int


def map_block(response):
    number_of_operations = count_operations(response.get('operations', EMPTY_LIST))

    header = response.get('header', EMPTY_OBJECT)
    metadata = response.get('metadata', EMPTY_OBJECT)
    level = metadata.get('level', EMPTY_OBJECT)

    block = {
        'item_type': 'block',
        'protocol': response.get('protocol'),
        'chain_id': response.get('chain_id'),
        'block_hash': response.get('hash'),
        'number_of_operation_groups': len(response.get('operations', EMPTY_LIST)),
        'number_of_operations': number_of_operations,
        'level': header.get('level'),
        'proto': header.get('proto'),
        'predecessor': header.get('predecessor'),
        'timestamp': header.get('timestamp'),
        'validation_pass': header.get('validation_pass'),
        'operations_hash': header.get('operations_hash'),
        'fitness': header.get('fitness'),
        'context': header.get('context'),
        'nonce_hash': metadata.get('nonce_hash'),
        'consumed_gas': safe_int(metadata.get('consumed_gas')),
        'baker': metadata.get('baker'),
        'voting_period_kind': metadata.get('voting_period_kind'),
        'cycle': level.get('cycle'),
        'cycle_position': level.get('cycle_position'),
        'voting_period': level.get('voting_period'),
        'voting_period_position': level.get('voting_period_position'),
        'expected_commitment': level.get('expected_commitment')
    }

    return block


def count_operations(operation_groups):
    number_of_operations = 0
    for operation_group in operation_groups:
        for operation in operation_group:
            number_of_operations += len(operation.get('contents', EMPTY_LIST))

            # Add internal operations
            for content in operation.get('contents', EMPTY_LIST):
                metadata = content.get('metadata', EMPTY_OBJECT)
                number_of_operations += len(metadata.get('internal_operation_results', EMPTY_LIST))

    return number_of_operations


EMPTY_OBJECT = {}
EMPTY_LIST = []



# MIT License
#
# Copyright (c) 2020 Evgeny Medvedev, evge.medvedev@gmail.com
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import logging
import os
import threading

from blockchainetl_common.atomic_counter import AtomicCounter
from blockchainetl_common.exporters import JsonLinesItemExporter, CsvItemExporter
from blockchainetl_common.file_utils import get_file_handle, close_silently


class TezosItemExporter:
    def __init__(self, output_dir, item_type_to_filename=None, output_format='json'):
        self.output_dir = output_dir
        self.item_type_to_filename = item_type_to_filename
        if self.item_type_to_filename is None:
            self.item_type_to_filename = lambda item_type: f'{item_type}s.{output_format}'
        self.output_format = output_format
        self.exporter_mapping = {}
        self.file_mapping = {}
        self.counter_mapping = {}
        self.init_lock = threading.Lock()

        self.logger = logging.getLogger('TezosItemExporter')

    def open(self):
        pass

    def export_items(self, items):
        for item in items:
            self.export_item(item)

    def export_item(self, item):
        item_type = item.get('item_type')
        if item_type is None:
            raise ValueError('"item_type" key is not found in item {}'.format(repr(item)))

        exporter = self._get_exporter_for_item_type(item_type)
        exporter.export_item(item)

        counter = self._get_counter_for_item_type(item_type)
        counter.increment()

    def _get_exporter_for_item_type(self, item_type):
        if self.exporter_mapping.get(item_type) is None:
            with self.init_lock:
                if self.exporter_mapping.get(item_type) is None:
                    filename = os.path.join(self.output_dir, self.item_type_to_filename(item_type))
                    file = get_file_handle(filename, binary=True)
                    self.file_mapping[item_type] = file
                    self.exporter_mapping[item_type] = get_item_exporter(self.output_format, file)
        return self.exporter_mapping[item_type]

    def _get_counter_for_item_type(self, item_type):
        if self.counter_mapping.get(item_type) is None:
            with self.init_lock:
                if self.counter_mapping.get(item_type) is None:
                    self.counter_mapping[item_type] = AtomicCounter()

        return self.counter_mapping[item_type]

    def close(self):
        for item_type, file in self.file_mapping.items():
            close_silently(file)
            counter = self.counter_mapping[item_type]
            if counter is not None:
                self.logger.info('{} items exported: {}'.format(item_type, counter.increment() - 1))


def get_item_exporter(output_format, file):
    if output_format == 'json':
        return JsonLinesItemExporter(file)
    elif output_format == 'csv':
        return CsvItemExporter(file)
    else:
        ValueError(f'output format {output_format} is not recognized')



# MIT License
#
# Copyright (c) 2020 Evgeny Medvedev, evge.medvedev@gmail.com
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import json

from tezosetl.enums.operation_kinds import OperationKind
from tezosetl.utils.cast_utils import safe_int


def map_operations(block, response):
    for operation_group_index, operation_index, operation in yield_operations(response):
        for content_index, content in enumerate(operation.get('contents', EMPTY_LIST)):
            operation_kind = content.get('kind')

            base_operation = map_base_operation(
                block, operation_group_index, operation_index, content_index, operation, operation_kind
            )

            yield map_operation(operation_kind, content, base_operation)

            metadata = content.get('metadata', EMPTY_OBJECT)
            for index, result in enumerate(metadata.get('internal_operation_results', EMPTY_LIST)):
                base_internal_operation = {**base_operation, **{
                    'internal_operation_index': index
                }}
                internal_operation_kind = result.get('kind')
                yield map_operation(internal_operation_kind, result, base_internal_operation)


def yield_operations(response):
    for operation_group_index, operation_group in enumerate(response.get('operations', EMPTY_LIST)):
        for operation_index, operation in enumerate(operation_group):
            yield operation_group_index, operation_index, operation


def map_base_operation(block, operation_group_index, operation_index, content_index, operation, operation_kind):
    return {
        'item_type': f'{operation_kind}_operation',
        'level': block.get('level'),
        'timestamp': block.get('timestamp'),
        'block_hash': block.get('block_hash'),
        'branch': operation.get('branch'),
        'signature': operation.get('signature'),
        'operation_hash': operation.get('hash'),
        'operation_group_index': operation_group_index,
        'operation_index': operation_index,
        'content_index': content_index,
        'internal_operation_index': None,
    }


def map_operation(operation_kind, content, base_operation):
    if operation_kind == OperationKind.endorsement:
        return map_endorsement(content, base_operation)
    elif operation_kind == OperationKind.transaction:
        return map_transaction(content, base_operation)
    elif operation_kind == OperationKind.delegation:
        return map_delegation(content, base_operation)
    elif operation_kind == OperationKind.reveal:
        return map_reveal(content, base_operation)
    elif operation_kind == OperationKind.seed_nonce_revelation:
        return map_seed_nonce_revelation(content, base_operation)
    elif operation_kind == OperationKind.activate_account:
        return map_activate_account(content, base_operation)
    elif operation_kind == OperationKind.origination:
        return map_origination(content, base_operation)
    elif operation_kind == OperationKind.proposals:
        return map_proposals(content, base_operation)
    elif operation_kind == OperationKind.double_baking_evidence:
        return map_double_baking_evidence(content, base_operation)
    elif operation_kind == OperationKind.double_endorsement_evidence:
        return map_double_endorsement_evidence(content, base_operation)
    elif operation_kind == OperationKind.ballot:
        return map_ballot(content, base_operation)
    else:
        raise KeyError(f'Operation kind {operation_kind} not recognized. {json.dumps(content)}')


def map_endorsement(content, base_operation):
    metadata = content.get('metadata', EMPTY_OBJECT)
    return {**base_operation, **{
        'delegate': metadata.get('delegate'),
        'public_key': content.get('public_key'),
        'slots': metadata.get('slots')
    }}


def map_transaction(content, base_operation):
    operation_result = get_operation_result(content)

    return {**base_operation, **{
        'source': content.get('source'),
        'destination': content.get('destination'),
        'fee': safe_int(content.get('fee')),
        'amount': safe_int(content.get('amount')),
        'counter': safe_int(content.get('counter')),
        'gas_limit': safe_int(content.get('gas_limit')),
        'storage_limit': safe_int(content.get('storage_limit')),
        'status': operation_result.get('status'),
        'consumed_gas': safe_int(operation_result.get('consumed_gas')),
        'storage_size': safe_int(operation_result.get('storage_size')),
        'parameters': json_dumps(content.get('parameters')),
    }}


def map_delegation(content, base_operation):
    operation_result = get_operation_result(content)

    return {**base_operation, **{
        'source': content.get('source'),
        'delegate': content.get('delegate'),
        'fee': safe_int(content.get('fee')),
        'counter': safe_int(content.get('counter')),
        'gas_limit': safe_int(content.get('gas_limit')),
        'storage_limit': safe_int(content.get('storage_limit')),
        'status': operation_result.get('status'),
    }}


def map_reveal(content, base_operation):
    operation_result = get_operation_result(content)

    return {**base_operation, **{
        'source': content.get('source'),
        'fee': safe_int(content.get('fee')),
        'counter': safe_int(content.get('counter')),
        'gas_limit': safe_int(content.get('gas_limit')),
        'storage_limit': safe_int(content.get('storage_limit')),
        'public_key': content.get('public_key'),
        'status': operation_result.get('status'),
    }}


def map_seed_nonce_revelation(content, base_operation):
    return {**base_operation, **{
        'nonce': content.get('nonce')
    }}


def map_activate_account(content, base_operation):
    return {**base_operation, **{
        'public_key_hash': content.get('pkh'),
        'secret': content.get('secret')
    }}


def map_origination(content, base_operation):
    operation_result = get_operation_result(content)

    return {**base_operation, **{
        'source': content.get('source'),
        'delegate': content.get('delegate'),
        'manager_public_key': content.get('managerPubkey'),
        'fee': safe_int(content.get('fee')),
        'counter': safe_int(content.get('counter')),
        'gas_limit': safe_int(content.get('gas_limit')),
        'storage_limit': safe_int(content.get('storage_limit')),
        'balance': safe_int(content.get('balance')),
        'status': operation_result.get('status'),
        'originated_contracts': operation_result.get('originated_contracts'),
        'script': json_dumps(content.get('script')),
    }}


def map_proposals(content, base_operation):
    return {**base_operation, **{
        'source': content.get('source'),
        'proposals': content.get('proposals'),
        'period': content.get('period'),
    }}


def map_double_baking_evidence(content, base_operation):
    bh1 = content.get('bh1', EMPTY_OBJECT)
    bh2 = content.get('bh2', EMPTY_OBJECT)
    return {**base_operation, **{
        'denounced_1_level': bh1.get('level'),
        'denounced_1_proto': bh1.get('proto'),
        'denounced_1_predecessor': bh1.get('predecessor'),
        'denounced_1_timestamp': bh1.get('timestamp'),
        'denounced_1_validation_pass': bh1.get('validation_pass'),
        'denounced_1_operations_hash': bh1.get('operations_hash'),
        'denounced_1_fitness': bh1.get('fitness'),
        'denounced_1_context': bh1.get('context'),
        'denounced_1_priority': bh1.get('priority'),
        'denounced_1_proof_of_work_nonce': bh1.get('proof_of_work_nonce'),
        'denounced_1_signature': bh1.get('signature'),
        'denounced_2_level': bh2.get('level'),
        'denounced_2_proto': bh2.get('proto'),
        'denounced_2_predecessor': bh2.get('predecessor'),
        'denounced_2_timestamp': bh2.get('timestamp'),
        'denounced_2_validation_pass': bh2.get('validation_pass'),
        'denounced_2_operations_hash': bh2.get('operations_hash'),
        'denounced_2_fitness': bh2.get('fitness'),
        'denounced_2_context': bh2.get('context'),
        'denounced_2_priority': bh2.get('priority'),
        'denounced_2_proof_of_work_nonce': bh2.get('proof_of_work_nonce'),
        'denounced_2_signature': bh2.get('signature'),
    }}


def map_double_endorsement_evidence(content, base_operation):
    op1 = content.get('op1', EMPTY_OBJECT)
    op2 = content.get('op2', EMPTY_OBJECT)
    return {**base_operation, **{
        'denounced_1_branch': op1.get('branch'),
        'denounced_1_signature': op1.get('signature'),
        'denounced_1_level': op1.get('operations', EMPTY_OBJECT).get('level'),
        'denounced_2_branch': op2.get('branch'),
        'denounced_2_signature': op2.get('signature'),
        'denounced_2_level': op2.get('operations', EMPTY_OBJECT).get('level'),
    }}


def map_ballot(content, base_operation):
    return {**base_operation, **{
        'source': content.get('source'),
        'proposal': content.get('proposal'),
        'period': content.get('period'),
        'ballot': content.get('ballot'),
    }}


def get_operation_result(content):
    metadata = content.get('metadata', EMPTY_OBJECT)
    if metadata.get('operation_result') is not None:
        return metadata.get('operation_result')
    elif content.get('result') is not None:
        return content.get('result')
    else:
        return EMPTY_OBJECT


def json_dumps(obj):
    if obj is None:
        return None
    return json.dumps(obj, separators=(',', ':'))


EMPTY_OBJECT = {}
EMPTY_LIST = []



# MIT License
#
# Copyright (c) 2020 Evgeny Medvedev evge.medvedev@gmail.com
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


import click

from tezosetl.jobs.export_job import ExportJob

from tezosetl.jobs.exporters.tezos_item_exporter import TezosItemExporter
from tezosetl.rpc.tezos_rpc import TezosRpc
from blockchainetl_common.logging_utils import logging_basic_config
from blockchainetl_common.thread_local_proxy import ThreadLocalProxy

logging_basic_config()


@click.command(context_settings=dict(help_option_names=['-h', '--help']))
@click.option('-s', '--start-block', default=0, show_default=True, type=int, help='Start block')
@click.option('-e', '--end-block', required=True, type=int, help='End block')
@click.option('-p', '--provider-uri', default='https://mainnet-tezos.giganode.io', show_default=True, type=str,
              help='The URI of the remote Tezos node')
@click.option('-w', '--max-workers', default=5, show_default=True, type=int, help='The maximum number of workers.')
@click.option('-o', '--output-dir', default=None, type=str, help='The output directory for block data.')
@click.option('-f', '--output-format', default='json', show_default=True, type=click.Choice(['json', 'csv']),
              help='The output format.')
def export(start_block, end_block, provider_uri, max_workers, output_dir, output_format):
    """Exports blocks, balance updates, and operations."""

    job = ExportJob(
        start_block=start_block,
        end_block=end_block,
        tezos_rpc=ThreadLocalProxy(lambda: TezosRpc(provider_uri)),
        max_workers=max_workers,
        item_exporter=TezosItemExporter(output_dir, output_format=output_format),
    )
    job.run()



# The MIT License (MIT)
#
# Copyright (c) 2020 Evgeny Medvedev, evge.medvedev@gmail.com
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import decimal
import json

from tezosetl.rpc.request import make_get_request


class TezosRpc:

    def __init__(self, provider_uri, timeout=60):
        self.provider_uri = provider_uri
        self.timeout = timeout

    def get(self, endpoint):
        raw_response = make_get_request(
            self.provider_uri + endpoint,
            timeout=self.timeout
        )

        response = self._decode_rpc_response(raw_response)
        return response

    def _decode_rpc_response(self, response):
        response_text = response.decode('utf-8')
        return json.loads(response_text, parse_float=decimal.Decimal)

    def getblock(self, block_id):
        return self.get('/chains/main/blocks/{}'.format(block_id))
